#include <Arduino.h>
#include "xtensa/core-macros.h"
#include <ESP32-HUB75-MatrixPanel-I2S-DMA.h>

#include "main.h"

#include "caveatbrush9pt7b.h"

#define R1 42
#define G1 40
#define BL1 41
#define R2 38
#define G2 37
#define BL2 39
#define CH_A 45
#define CH_B 36
#define CH_C 48
#define CH_D 35
#define CH_E 21
#define CLK 2
#define LAT 47
#define OE 14

#define PIN_E 21
#define PANEL_WIDTH 64
#define PANEL_HEIGHT 32         // Panel height of 64 will required PIN_E to be defined.

#define PANELS_NUMBER 1

#define PANE_WIDTH PANEL_WIDTH * PANELS_NUMBER
#define PANE_HEIGHT PANEL_HEIGHT
#define NUM_LEDS PANE_WIDTH*PANE_HEIGHT

#define ONBOARD_LED 13

MatrixPanel_I2S_DMA *matrix = nullptr;

uint16_t colorWheel(uint8_t pos) {
  if(pos < 85) {
    return matrix->color565(pos * 3, 255 - pos * 3, 0);
  } else if(pos < 170) {
    pos -= 85;
    return matrix->color565(255 - pos * 3, 0, pos * 3);
  } else {
    pos -= 170;
    return matrix->color565(0, pos * 3, 255 - pos * 3);
  }
}

int line_pos[PANEL_WIDTH]; // where a rain marker is (y coordinate) 
int line_length[PANEL_WIDTH]; // how long a line is for a given x coord
int line_speed[PANEL_WIDTH]; // how fast each line moves

void setup(){
  Serial.begin(BAUD_RATE);
  pinMode(ONBOARD_LED, OUTPUT);

  for (int i = 0; i<PANEL_WIDTH; i++) {
    line_pos[i] = 0 - random(32);
    line_length[i] = random(5, 12);
    line_speed[i] = random(10, 100);
  }
  // redefine pins if required
  HUB75_I2S_CFG::i2s_pins _pins={R1, G1, BL1, R2, G2, BL2, CH_A, CH_B, CH_C, CH_D, CH_E, LAT, OE, CLK};
  HUB75_I2S_CFG mxconfig(PANEL_WIDTH, PANEL_HEIGHT, PANELS_NUMBER, _pins);

  mxconfig.gpio.e = PIN_E;
  mxconfig.driver = HUB75_I2S_CFG::FM6126A;   // for panels using FM6126A chips

  mxconfig.latch_blanking = 4;
  mxconfig.i2sspeed = HUB75_I2S_CFG::HZ_10M;
  mxconfig.clkphase = false;

  matrix = new MatrixPanel_I2S_DMA(mxconfig);
  matrix->begin();
  matrix->setBrightness8(64);
  matrix->fillScreenRGB888(0, 0, 0);
  matrix->setFont(&CaveatBrush_Regular9pt7b);
}

uint8_t color = 0;
const char *str = "Panki";
void loop() {

  matrix->fillScreenRGB888(0, 0, 0);
  for (int x = 0; x < PANEL_WIDTH; x++) {
    // step down a pixel
    line_pos[x]++;
    if (line_pos[x] > PANEL_HEIGHT) {
      line_pos[x] = 0;
    }
    if (line_pos[x] >= 0) {
      //draw our pixel
      matrix->drawPixel(x, line_pos[x], matrix->color565(128, 255, 128));
      // draw our trail
      for (int trail_offset = 1; trail_offset < line_length[x]; trail_offset++) {
        //if (random(10) > 8) {
        //  trail_offset++;
        //}
        int trail_y = line_pos[x] - trail_offset;
        if (trail_y >= 0) {
          matrix->drawPixel(x, trail_y, matrix->color565(0, (255 / line_length[x]) * (line_length[x] - trail_offset), 0));
        } else {
          if (line_pos[x] >= 0) {
            matrix->drawPixel(x, PANEL_HEIGHT + trail_y, matrix->color565(0, 255 - (trail_offset * 16) , 0));
          } else {
            break;
          }
        }
      }
    }
  }
  delay(100);
  //for (color=0; color<256; color++) {
  //  // create a border
  //  matrix->drawRect(0, 0, matrix->width(), matrix->height(), colorWheel(color));
  //  matrix->drawRect(1, 1, matrix->width()-2, matrix->height()-2, colorWheel(color));

  //  matrix->setTextSize(1);
  //  matrix->setTextWrap(false);
  //  matrix->setCursor(8, 22);

  //  //const char *str = "Panki";
  //  matrix->setTextColor(colorWheel(color+128));
  //  matrix->print(str);
  //  delay(100);
  //}
}

